@@ -39,7 +39,7 @@ class DeviceSelectorList(DeviceList):
     def row_setup_event(self, tree_iter: Gtk.TreeIter, device: Device) -> None:
         self.row_update_event(tree_iter, "Trusted", device['Trusted'])
         self.row_update_event(tree_iter, "Paired", device['Paired'])
-        self.row_update_event(tree_iter, "Alias", device['Alias'])
+        self.row_update_event(tree_iter, "Alias", device.display_name)
         self.row_update_event(tree_iter, "Icon", device['Icon'])
 
     def row_update_event(self, tree_iter: Gtk.TreeIter, key: str, value: Any) -> None:
