@@ -1,8 +1,10 @@
-from typing import Iterable, TYPE_CHECKING, Callable, List, Tuple, Dict, Union, TypeVar
+from collections import OrderedDict
+from typing import Iterable, TYPE_CHECKING, Callable, List, Tuple, Dict, Union, TypeVar, Any
 
 from gi.repository import Gio, GLib, Pango
 
 from blueman.main.DbusService import DbusService
+from blueman.main.Tray import BluemanTray
 from blueman.main.indicators.IndicatorInterface import IndicatorInterface, IndicatorNotAvailable
 
 if TYPE_CHECKING:
@@ -13,13 +15,14 @@ if TYPE_CHECKING:
 class MenuService(DbusService):
     def __init__(self, on_activate_menu_item: "MenuItemActivator") -> None:
         super().__init__(None, "com.canonical.dbusmenu", "/org/blueman/sni/menu", Gio.BusType.SESSION)
-        self._items: List["MenuItemDict"] = []
+        self._items: OrderedDict[int, "MenuItemDict"] = OrderedDict()
         self._revision = 0
         self._revision_advertised = -1
         self._on_activate = on_activate_menu_item
 
         self.add_method("GetLayout", ("i", "i", "as"), ("u", "(ia{sv}av)"), self._get_layout)
         self.add_method("Event", ("i", "s", "v", "u"), (), self._on_event)
+        self.add_method("AboutToShow", ("i",), ("b",), lambda _: self._revision > self._revision_advertised)
 
         self.add_method("GetGroupProperties", ("ai", "as"), ("a(ia{sv})",),
                         lambda ids, props: [(idx, self._render_item(item)) for idx, item in self._iterate_items()
@@ -30,7 +33,7 @@ class MenuService(DbusService):
         GLib.timeout_add(100, self._advertise_revision)
 
     def set_items(self, items: Iterable["MenuItemDict"]) -> None:
-        self._items = list(items)
+        self._items = OrderedDict((item["id"], item) for item in items)
         self._revision += 1
 
     def _advertise_revision(self) -> bool:
@@ -42,31 +45,32 @@ class MenuService(DbusService):
     def _get_layout(self, parent_id: int, _recursion_depth: int, _property_names: List[str]
                     ) -> Tuple[int, Tuple[int, Dict[str, GLib.Variant], List[GLib.Variant]]]:
         if parent_id == 0:
-            return self._revision, (0, {}, self._render_menu(self._items, 1, self._render_submenu))
+            return self._revision, (0, {}, self._render_menu(((item["id"] << 8, item) for item in self._items.values()),
+                                                             self._render_submenu))
         else:
-            item = self._items[parent_id - 1]
+            item = self._items[parent_id >> 8]
             if "submenu" in item and _recursion_depth != 0:
                 return self._revision, (parent_id, self._render_item(item), self._render_submenu(item, parent_id))
             return self._revision, (parent_id, self._render_item(item), [])
 
     def _render_submenu(self, item: "MenuItemDict", idx: int) -> List[GLib.Variant]:
         if "submenu" in item:
-            return self._render_menu(item["submenu"], idx * 100 + 1, lambda _item, _isx: [])
+            return self._render_menu(enumerate(item["submenu"], idx + 1), lambda _item, _isx: [])
         else:
             return []
 
     _T = TypeVar("_T", bound="SubmenuItemDict")
 
-    def _render_menu(self, items: Iterable[_T], start: int, submenu_callback: Callable[[_T, int], List[GLib.Variant]]
+    def _render_menu(self, items: Iterable[Tuple[int, _T]], submenu_callback: Callable[[_T, int], List[GLib.Variant]]
                      ) -> List[GLib.Variant]:
         return [GLib.Variant("(ia{sv}av)", (idx, self._render_item(item), submenu_callback(item, idx)))
-                for idx, item in enumerate(items, start)]
+                for (idx, item) in items]
 
     def _iterate_items(self) -> Iterable[Tuple[int, "SubmenuItemDict"]]:
-        for idx, item in enumerate(self._items, 1):
-            yield idx, item
+        for item in self._items.values():
+            yield item["id"] << 8, item
             if "submenu" in item:
-                yield from enumerate(item["submenu"], idx * 100 + 1)
+                yield from enumerate(item["submenu"], (item["id"] << 8) + 1)
 
     def _render_item(self, item: Union["MenuItemDict", "SubmenuItemDict"]) -> Dict[str, GLib.Variant]:
         if "text" in item and "icon_name" in item:
@@ -84,10 +88,10 @@ class MenuService(DbusService):
 
     def _on_event(self, idx: int, event_id: str, _data: GLib.Variant, _timestamp: int) -> None:
         if event_id == "clicked":
-            if idx < 100:
-                self._on_activate(idx - 1)
+            if idx % (1 << 8) == 0:
+                self._on_activate(idx >> 8)
             else:
-                self._on_activate(int(idx / 100) - 1, idx % 100 - 1)
+                self._on_activate(idx >> 8, idx % (1 << 8) - 1)
 
 
 class StatusNotifierItemService(DbusService):
@@ -96,14 +100,13 @@ class StatusNotifierItemService(DbusService):
     Title = "blueman"
     ItemIsMenu = False
 
-    def __init__(self, icon_name: str, on_activate_status_icon: Callable[[], None],
-                 on_activate_menu_item: "MenuItemActivator") -> None:
+    def __init__(self, tray: BluemanTray, icon_name: str) -> None:
         super().__init__(None, "org.kde.StatusNotifierItem", "/org/blueman/sni", Gio.BusType.SESSION,
                          {"Category": "s", "Id": "s", "IconName": "s", "Status": "s", "Title": "s",
                           "ToolTip": "(sa(iiay)ss)", "Menu": "o", "ItemIsMenu": "b"})
-        self.add_method("Activate", ("i", "i"), "", lambda x, y: on_activate_status_icon())
+        self.add_method("Activate", ("i", "i"), "", lambda x, y: tray.activate_status_icon())
 
-        self.menu = MenuService(on_activate_menu_item)
+        self.menu = MenuService(tray.activate_menu_item)
 
         self.IconName = icon_name
         self.Status = "Active"
@@ -126,27 +129,35 @@ class StatusNotifierItemService(DbusService):
 class StatusNotifierItem(IndicatorInterface):
     _SNI_BUS_NAME = _SNI_INTERFACE_NAME = "org.kde.StatusNotifierWatcher"
 
-    def __init__(self, icon_name: str, on_activate_menu_item: "MenuItemActivator",
-                 on_activate_status_icon: Callable[[], None]) -> None:
-        self._sni = StatusNotifierItemService(icon_name, on_activate_status_icon, on_activate_menu_item)
+    def __init__(self, tray: BluemanTray, icon_name: str) -> None:
+        self._sni = StatusNotifierItemService(tray, icon_name)
         self._sni.register()
 
         self._bus = Gio.bus_get_sync(Gio.BusType.SESSION)
 
+        watcher_expected: bool
+
+        def on_watcher_appeared(*args: Any) -> None:
+            nonlocal watcher_expected
+
+            if watcher_expected:
+                watcher_expected = False
+            else:
+                tray.activate()
+
         Gio.bus_watch_name(Gio.BusType.SESSION, self._SNI_BUS_NAME, Gio.BusNameWatcherFlags.NONE,
-                           lambda *args: self._register(), None)
+                           on_watcher_appeared, None)
 
         try:
-            self._register()
-        except GLib.Error:
+            Gio.bus_get_sync(Gio.BusType.SESSION).call_sync(
+                self._SNI_BUS_NAME, "/StatusNotifierWatcher", self._SNI_INTERFACE_NAME,
+                "RegisterStatusNotifierItem", GLib.Variant("(s)", ("/org/blueman/sni",)),
+                None, Gio.DBusCallFlags.NONE, -1)
+            watcher_expected = True
+        except GLib.Error as e:
+            watcher_expected = not e.message.startswith("org.freedesktop.DBusError.ServiceUnknown")
             raise IndicatorNotAvailable
 
-    def _register(self) -> None:
-        Gio.bus_get_sync(Gio.BusType.SESSION).call_sync(
-            self._SNI_BUS_NAME, "/StatusNotifierWatcher", self._SNI_INTERFACE_NAME,
-            "RegisterStatusNotifierItem", GLib.Variant("(s)", ("/org/blueman/sni",)),
-            None, Gio.DBusCallFlags.NONE, -1)
-
     def set_icon(self, icon_name: str) -> None:
         self._sni.IconName = icon_name
         self._sni.emit_signal("NewIcon")
