@@ -1,18 +1,20 @@
 from gettext import gettext as _
 from operator import itemgetter
+import html
 import time
 import logging
 from typing import List, TYPE_CHECKING, Optional, Callable, cast, Union
 
 from blueman.bluez.Device import Device
-from blueman.bluez.errors import DBusNoSuchAdapterError
+from blueman.bluez.errors import DBusNoSuchAdapterError, DBusUnknownObjectError
 from blueman.gui.Notification import Notification
 from blueman.Sdp import ServiceUUID
 from blueman.plugins.AppletPlugin import AppletPlugin
+from blueman.plugins.applet.Menu import MenuItem
 from blueman.plugins.applet.PowerManager import PowerManager, PowerStateListener
 
 if TYPE_CHECKING:
-    from blueman.plugins.applet.Menu import SubmenuItemDict
+    from blueman.plugins.applet.Menu import Menu, SubmenuItemDict
 
     from typing_extensions import TypedDict
 
@@ -26,7 +28,7 @@ if TYPE_CHECKING:
 
     class Item(_ItemBase):
         time: float
-        device: Optional[str]
+        device: str
         mitem: Optional[SubmenuItemDict]
 
     class StoredIcon(_ItemBase):
@@ -55,15 +57,10 @@ class RecentConns(AppletPlugin, PowerStateListener):
         "recent-connections": {"type": list, "default": "[]"}
     }
 
-    _items = None
-
     def on_load(self) -> None:
         self.__menuitems: List["SubmenuItemDict"] = []
 
-        self._item = self.parent.Plugins.Menu.add(self, 52, text=_("Recent _Connections") + "…",
-                                                  icon_name="document-open-recent-symbolic",
-                                                  submenu_function=self.get_menu)
-        self.parent.Plugins.Menu.add(self, 53)
+        self._rebuild_menu()
 
     def on_power_state_changed(self, manager: PowerManager, state: bool) -> None:
         self._rebuild()
@@ -74,26 +71,30 @@ class RecentConns(AppletPlugin, PowerStateListener):
     def _rebuild(self) -> None:
         if 'PowerManager' in self.parent.Plugins.get_loaded() and \
                 not self.parent.Plugins.PowerManager.get_bluetooth_status():
-            self._item.set_sensitive(False)
+            for mitem in self._mitems:
+                mitem.set_sensitive(False)
             return
 
-        self._items = self._get_items()
+        items = self._get_items()
 
-        if len(self._items) == 0:
-            self._item.set_sensitive(False)
+        if len(items) == 0:
+            for mitem in self._mitems:
+                mitem.set_sensitive(False)
             return
 
-        self._item.set_sensitive(True)
+        for mitem in self._mitems:
+            mitem.set_sensitive(True)
 
-        self.__menuitems = [self._build_menu_item(item) for item in self._items[:self.get_option("max-items")]]
+        self.__menuitems = [self._build_menu_item(item) for item in items[:self.get_option("max-items")]]
 
-        self.parent.Plugins.Menu.on_menu_changed()
+        self._rebuild_menu()
 
     def on_manager_state_changed(self, state: bool) -> None:
         if state:
             self._rebuild()
         else:
-            self._item.set_sensitive(False)
+            for mitem in self._mitems:
+                mitem.set_sensitive(False)
 
     def on_device_created(self, path: str) -> None:
         self._rebuild()
@@ -119,7 +120,7 @@ class RecentConns(AppletPlugin, PowerStateListener):
         item = {
             "adapter": adapter["Address"],
             "address": device['Address'],
-            "alias": device['Alias'],
+            "alias": device.display_name,
             "icon": device['Icon'],
             "name": ServiceUUID(uuid).name,
             "uuid": uuid,
@@ -167,8 +168,9 @@ class RecentConns(AppletPlugin, PowerStateListener):
         self.parent.Plugins.DBusService.connect_service(item["device"], item["uuid"], reply, err)
 
     def _build_menu_item(self, item: "Item") -> "SubmenuItemDict":
+        alias = html.escape(item["alias"])
         mitem: "SubmenuItemDict" = {
-            "text": _("%(service)s on %(device)s") % {"service": item["name"], "device": item["alias"]},
+            "text": _("%(service)s on %(device)s") % {"service": item["name"], "device": alias},
             "markup": True,
             "icon_name": item["mitem"]["icon_name"] if item["mitem"] is not None else item["icon"],
             "sensitive": item["device"] is not None,
@@ -181,8 +183,15 @@ class RecentConns(AppletPlugin, PowerStateListener):
 
         return mitem
 
-    def get_menu(self) -> List["SubmenuItemDict"]:
-        return self.__menuitems
+    def _rebuild_menu(self) -> None:
+        menu: "Menu" = self.parent.Plugins.Menu
+        self._mitems: List[MenuItem] = []
+        menu.unregister(self)
+        menu.add(self, 52, text=_("Reconnect to…"), icon_name="document-open-recent-symbolic",
+                 sensitive=False, callback=lambda: None)
+        for (idx, item) in enumerate(self.__menuitems):
+            self._mitems.append(menu.add(self, (53, idx), **item))
+        menu.add(self, 59)
 
     def _get_device_path(self, adapter_path: str, address: str) -> Optional[str]:
         try:
@@ -190,12 +199,14 @@ class RecentConns(AppletPlugin, PowerStateListener):
         except DBusNoSuchAdapterError:
             return None
 
-        device = self.parent.Manager.find_device(address, adapter.get_object_path())
+        try:
+            device = self.parent.Manager.find_device(address, adapter.get_object_path())
+        except DBusUnknownObjectError:
+            return None
+
         return device.get_object_path() if device is not None else None
 
     def _get_items(self) -> List["Item"]:
-        adapter_addresses = {adapter["Address"] for adapter in self.parent.Manager.get_adapters()}
-
         return sorted(
             ({
                 "adapter": i["adapter"],
@@ -205,12 +216,11 @@ class RecentConns(AppletPlugin, PowerStateListener):
                 "name": i["name"],
                 "uuid": i["uuid"],
                 "time": float(i["time"]),
-                "device": (self._get_device_path(i["adapter"], i["address"])
-                           if i["adapter"] in adapter_addresses else None),
+                "device": device,
                 "mitem": None
             }
                 for i in self.get_option("recent-connections")
-                if i["adapter"] not in adapter_addresses or self._get_device_path(i["adapter"], i["address"])),
+                if (device := self._get_device_path(i["adapter"], i["address"]))),
             key=itemgetter("time"),
             reverse=True
         )
