@@ -6,7 +6,7 @@ from blueman.gui.Notification import Notification
 from blueman.Sdp import SERIAL_PORT_SVCLASS_ID
 from blueman.plugins.applet.DBusService import RFCOMMConnectedListener
 from blueman.services.Functions import get_services
-from _blueman import rfcomm_list
+from _blueman import rfcomm_list, RFCOMMError
 from subprocess import Popen
 import logging
 import os
@@ -33,7 +33,7 @@ class SerialManager(AppletPlugin, RFCOMMConnectedListener):
                              "Address, Name, service name, uuid16s, rfcomm node\n"
                              "For example:\n"
                              "AA:BB:CC:DD:EE:FF, Phone, DUN service, 0x1103, /dev/rfcomm0\n"
-                             "uuid16s are returned as a comma seperated list\n\n"
+                             "uuid16s are returned as a comma separated list\n\n"
                              "Upon device disconnection the script will be sent a HUP signal</span>")},
     }
 
@@ -62,11 +62,11 @@ class SerialManager(AppletPlugin, RFCOMMConnectedListener):
         if SERIAL_PORT_SVCLASS_ID == service.short_uuid:
             Notification(_("Serial port connected"),
                          _("Serial port service on device <b>%s</b> now will be available via <b>%s</b>") % (
-                         device['Alias'], port),
+                         device.display_name, port),
                          icon_name="blueman-serial").show()
 
             self.call_script(device['Address'],
-                             device['Alias'],
+                             device.display_name,
                              service.name,
                              service.short_uuid,
                              port)
@@ -117,9 +117,8 @@ class SerialManager(AppletPlugin, RFCOMMConnectedListener):
                              icon_name="blueman-serial").show()
 
     def on_rfcomm_disconnect(self, port: int) -> None:
-        node = '/dev/rfcomm%i' % port
         for bdaddr, scripts in self.scripts.items():
-            process = scripts.get(node)
+            process = scripts.get(f"/dev/rfcomm{port:i}")
             if process:
                 logging.info(f"Sending HUP to {process.pid}")
                 os.killpg(process.pid, signal.SIGHUP)
@@ -130,10 +129,14 @@ class SerialManager(AppletPlugin, RFCOMMConnectedListener):
         if not serial_services:
             return
 
-        active_ports = [rfcomm['id'] for rfcomm in rfcomm_list() if rfcomm['dst'] == device['Address']]
+        try:
+            active_ports = [rfcomm['id'] for rfcomm in rfcomm_list() if rfcomm['dst'] == device['Address']]
+        except RFCOMMError as e:
+            logging.error(f"rfcomm_list failed with: {e}")
+            return
 
         for port in active_ports:
-            name = "/dev/rfcomm%d" % port
+            name = f"/dev/rfcomm{port:d}"
             try:
                 logging.info(f"Disconnecting {name}")
                 serial_services[0].disconnect(port)
