@@ -12,7 +12,11 @@ Gdk.Screen.get_default = lambda: fake_x11_screen
 class TestImports(TestCase):
     def __init__(self, mod_name):
         name = f"test_{mod_name.replace('.', '_')}_import"
-        setattr(self, name, lambda: __import__(mod_name))
+
+        def f():
+            __import__(mod_name)
+
+        setattr(self, name, f)
         super().__init__(name)
 
 
